/**************************** Ejemplo de programacion con cryptlib ****************************/
/*********************************** Cifrado simetrico ****************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
//biblioteca para cryptlib
#include "cryptlib.h"

#define MAXMSJ 1024
#define MAXPSWD 16
#define SEMILLA "12345678"
#define IV "87654321"
#define ALGORITMO CRYPT_ALGO_3DES
#define MODO CRYPT_MODE_CFB

//estructura para intercambio de parametros
struct datos{
  char clave_sesion[128];
  char msj_cifrado[MAXMSJ];  
};
typedef struct datos datos;

//definicion de funciones de cifrado y descifrado
int cifra(datos *mensaje);
int descifra(datos mensaje);


/******* Funcion principal *********/

int main(int argc, char *argv[]){

  datos mensaje;

  system("clear");

  printf("Aplicacion que cifra un mensaje con cifrado simetrico\n");
  printf("-----------------------------------------------------\n");

  cifra(&mensaje);

  descifra(mensaje);

  exit(0);

}



/********************************** Funcion de cifrado ***********************/
int cifra(datos *mensaje){

  int n, status, encryptedKeyLength;
  char password[MAXPSWD];
  char msj[MAXMSJ];

  //Estructuras para trabajo con cryptlib
  CRYPT_CONTEXT masterContext;
  CRYPT_CONTEXT sessionContext;
  

  /************* pide los valores de entrada al usuario **************/

  fflush(stdin);
  printf("\n\nFUNCION DE CIFRADO\n");

  //Mensaje a cifrar
  printf("\nIntroduzca el mensaje a cifrar:\n");
  fgets(msj,MAXMSJ,stdin);
  fflush(stdin);
  n=strlen(msj);
  msj[n]=0;

  //Contrasea
  printf("\nIntroduzca su contrasea secreta:\t");
  scanf("%s", password);
  n=strlen(password);
  password[n]=0;
  fflush(stdin);

  
  /********************************************************************
   ********************** Cifrado de los datos ************************
   ********************************************************************/
  
  /************** Inicio del cifrado ************/
  
  status=cryptInit();
  if(cryptStatusError(status))
    printf("Init: %d\n", status);
  
  
  /************** Contexto Master ***************/

  //Creacion del contexto: depende del algoritmo de cifrado que queramos utilizar, 
  //en este caso DES para cifrar la clave de sesion
  status=cryptCreateContext(&masterContext, CRYPT_UNUSED, CRYPT_ALGO_DES);
  if(cryptStatusError(status))
    printf("Crear contexto master: %d\n", status);

  //Derivacion de claves: obtener clave maestra a partir de la password del usuario
  //se introduce la semilla
  status=cryptSetAttributeString(masterContext, CRYPT_CTXINFO_KEYING_SALT, SEMILLA, 8);
  if(cryptStatusError(status))
  printf("semilla: %d\n", status);
  
  //se introduce la contrasea
  status=cryptSetAttributeString(masterContext, CRYPT_CTXINFO_KEYING_VALUE, password, strlen(password));
  if(cryptStatusError(status))
    printf("Deriva clave: %d\n", status);

  //se introduce el vector de inicializacion si es necesario. En este caso lo es porque usamos
  //el modo por defecto (CBC), que lo necesita
  status=cryptSetAttributeString(masterContext, CRYPT_CTXINFO_IV, IV, 8);
  if(cryptStatusError(status))
    printf("IV: %d\n", status);
  

  /*************** Contexto de sesion *************/

  //Generacion del contexto, dependera del algoritmo que hayamos seleccionado
  status=cryptCreateContext(&sessionContext, CRYPT_UNUSED, ALGORITMO);
  if(cryptStatusError(status))
    printf("Crear contexto de sesion: %d\n", status);
    
  //Seleccion del modo de operacion
  status=cryptSetAttribute(sessionContext, CRYPT_CTXINFO_MODE, MODO);
  if(cryptStatusError(status))
    printf("Modo de operacion contexto de sesion: %d\n", status);

  //Generacion de clave de sesion de forma aleatoria
  status=cryptGenerateKey(sessionContext);
  if(cryptStatusError(status))
    printf("Generar clave de sesion: %d\n", status);
  
  //vector de inicializacion si es necesario (en todos los metodos menos ECB)
  status=cryptSetAttributeString(sessionContext, CRYPT_CTXINFO_IV, IV, 8);
  if(cryptStatusError(status))
    printf("IV: %d\n", status);

  
  /** Exportacion de clave del contexto master al contexto de sesion **/

  //Calculo de la memoria necesaria para la exportacion
  status=cryptExportKey(NULL, &encryptedKeyLength, masterContext, sessionContext);
  if(cryptStatusError(status))
    printf("Calculo de memoria para exportacion de clave: %d\n", status);

  //Exportacion
  status=cryptExportKey(mensaje->clave_sesion, &encryptedKeyLength, masterContext, sessionContext);
  if(cryptStatusError(status))
    printf("Exportacion de clave: %d\n", status);

  
  /************* Cifrado de los datos ****************/
  
  //Cifra el contenido de msj con la clave y metodos asociados al contexto de sesion
  //y lo vuelve a guardar en msj
  n=strlen(msj);

  //  while(n%8)
  //    n=n+1;

  status=cryptEncrypt(sessionContext, msj, n);
  if(cryptStatusError(status))
    printf("Cifrado de datos: %d\n", status);

  
  /************ Destruccion de contextos *****************/

  status=cryptDestroyContext(sessionContext);
  if(cryptStatusError(status))
    printf("Destruccion contexto de sesion: %d\n", status);

  status=cryptDestroyContext(masterContext);
  if(cryptStatusError(status))
    printf("destruccion de contexto master: %d\n", status);

  /************* Fin del cifrado ***************/

  status=cryptEnd();
  if(cryptStatusError(status))
    printf("End: %d\n", status);


  
  /****************************************************************
   ****** Asignacion de parametros a la estructura de datos *******
   ****************************************************************/

  //mensaje
  strcpy(mensaje->msj_cifrado, msj);
  n=strlen(msj);
  mensaje->msj_cifrado[n]='\0';


  //presentacion de datos por pantalla
  printf("\n\nMensaje cifrado: %s\n", msj);


  return(0);


}




/************************ Funcion de descifrado *************************/
int descifra(datos mensaje){

  int n, status;
  char password[MAXPSWD];
  char msj[MAXMSJ];
  //  char iv[9];

  //Estructuras para trabajo con cryptlib
  CRYPT_CONTEXT masterContext;
  CRYPT_CONTEXT sessionContext;


  /************* pide los valores de entrada al usuario **************/

  fflush(stdin);
  printf("\n\n\n-------------------------------------------------");
  printf("\n\nFUNCION DE DESCIFRADO\n");

  //Contrasea
  printf("\nIntroduzca su contrasea secreta:\t");
  scanf("%s", password);
  n=strlen(password);
  password[n]=0;

  
  /********************************************************************
   ******* Asignacion de parametros de la estructura de datos *********
   ********************************************************************/

  strcpy(msj, mensaje.msj_cifrado);



  /********************************************************************
   ********************** Descifrado de los datos *********************
   ********************************************************************/

  /************** Inicio del descifrado ************/

  status=cryptInit();
  if(cryptStatusError(status))
    printf("Init: %d\n", status);


  /************** Contexto Master ***************/

  //Creacion del contexto: depende del algoritmo de cifrado que queramos utilizar,
  //en este caso DES para cifrar la clave de sesion
  status=cryptCreateContext(&masterContext, CRYPT_UNUSED, CRYPT_ALGO_DES);
  if(cryptStatusError(status))
    printf("Crear contexto master: %d\n", status);

  //Derivacion de claves: obtener clave maestra a partir de la password del usuario
  //se introduce la semilla
  status=cryptSetAttributeString(masterContext, CRYPT_CTXINFO_KEYING_SALT, SEMILLA, 8);
  if(cryptStatusError(status))
  printf("semilla: %d\n", status);

  //se introduce la contrasea
  status=cryptSetAttributeString(masterContext, CRYPT_CTXINFO_KEYING_VALUE, password, strlen(password));
  if(cryptStatusError(status))
    printf("Deriva clave: %d\n", status);

  //se introduce el vector de inicializacion si es necesario. En este caso lo es porque usamos
  //el modo por defecto (CBC), que lo necesita
  status=cryptSetAttributeString(masterContext, CRYPT_CTXINFO_IV, IV, 8);
  if(cryptStatusError(status))
    printf("IV: %d\n", status);

  
  /*************** Contexto de sesion *************/

  //Generacion del contexto, dependera del algoritmo que hayamos seleccionado
  status=cryptCreateContext(&sessionContext, CRYPT_UNUSED, ALGORITMO);
  if(cryptStatusError(status))
    printf("Crear contexto de sesion: %d\n", status);

  //Seleccion del modo de operacion
  status=cryptSetAttribute(sessionContext, CRYPT_CTXINFO_MODE, MODO);
  if(cryptStatusError(status))
    printf("Modo de operacion contexto de sesion: %d\n", status);


  /** Importacion de clave del contexto master al contexto de sesion **/
  status=cryptImportKey(mensaje.clave_sesion, masterContext, sessionContext);
  if(cryptStatusError(status))
    printf("Importacion de clave: %d\n", status);


  //vector de inicializacion si es necesario (en todos los metodos menos ECB)
  status=cryptSetAttributeString(sessionContext, CRYPT_CTXINFO_IV, IV, 8);
  if(cryptStatusError(status))
    printf("IV: %d\n", status);


  /************* Descifrado de los datos ****************/

  //Descifra el contenido de msj con la clave y metodos asociados al contexto de sesion
  //y lo vuelve a guardar en msj
  n=strlen(msj);

  status=cryptDecrypt(sessionContext, msj, n);
  if(cryptStatusError(status))
    printf("Descifrado de datos: %d\n", status);


  /************ Destruccion de contextos *****************/

  status=cryptDestroyContext(sessionContext);
  if(cryptStatusError(status))
    printf("Destruccion contexto de sesion: %d\n", status);

  status=cryptDestroyContext(masterContext);
  if(cryptStatusError(status))
    printf("destruccion de contexto master: %d\n", status);


  /************* Fin del descifrado ***************/

  status=cryptEnd();
  if(cryptStatusError(status))
    printf("End: %d\n", status);


  //presentacion de datos por pantalla
  printf("\n\nMensaje descifrado: %s\n\n", msj);
  
  return(0);
}
