/**************************** Ejemplo de programacion con cryptlib ****************************/
/*********************************** Cifrado simetrico ****************************************/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
//biblioteca para cryptlib
#include "cryptlib.h"

#define MAXMSJ 1024
#define MAXPSWD 16
#define SEMILLA "12345678"
#define IV "87654321"
#define ALGORITMO CRYPT_ALGO_3DES
#define MODO CRYPT_MODE_CBC

//estructura para intercambio de parametros
struct datos{
  char clave_sesion[128];
  char msj_cifrado[MAXMSJ];  
};
typedef struct datos datos;

//definicion de funciones de cifrado y descifrado
int cifra(datos *mensaje);
int descifra(datos mensaje);


/******* Funcion principal *********/
int main(int argc, char *argv[]){

  datos mensaje;

  system("clear");

  printf("Aplicacion que cifra un mensaje con cifrado simetrico\n");
  printf("-----------------------------------------------------\n");

  cifra(&mensaje);

  descifra(mensaje);

  exit(0);

}


/********************************** Funcion de cifrado ***********************/
int cifra(datos *mensaje){

  int n, status, encryptedKeyLength;
  char password[MAXPSWD];
  char msj[MAXMSJ];
  //Estructuras para trabajo con cryptlib
  CRYPT_CONTEXT masterContext;
  CRYPT_CONTEXT sessionContext;

  /************* valores de entrada **************/
  printf("\n\nFUNCION DE CIFRADO\n");

  //Mensaje a cifrar
  printf("\nIntroduzca el mensaje a cifrar:\n");
  fgets(msj,MAXMSJ,stdin);
  n=strlen(msj);
  msj[n]=0;

  //Contrasea
  printf("\nIntroduzca su contrasea secreta:\t");
  scanf("%s", password);
  n=strlen(password);
  password[n]=0;


  /************** Inicio del cifrado ************/
  status=cryptInit();
  if(cryptStatusError(status))
    printf("Init: %d\n", status);
  
  
  /************** Contexto Master ***************/

  //Creacion del contexto: depende del algoritmo de cifrado que queramos utilizar, 
  //en este caso DES para cifrar la clave de sesion
  cryptCreateContext(&masterContext, CRYPT_UNUSED, CRYPT_ALGO_DES);

  //Derivacion de claves: obtener clave maestra a partir de la password del usuario
  //se introduce la semilla
  cryptSetAttributeString(masterContext, CRYPT_CTXINFO_KEYING_SALT, SEMILLA, 8);
  
  //se introduce la contrasea
  cryptSetAttributeString(masterContext, CRYPT_CTXINFO_KEYING_VALUE, password, strlen(password));

  //se introduce el vector de inicializacion si es necesario. En este caso lo es porque usamos
  //el modo por defecto (CBC), que lo necesita
  cryptSetAttributeString(masterContext, CRYPT_CTXINFO_IV, IV, 8);
  
  
  /*************** Contexto de sesion *************/

  //Generacion del contexto, dependera del algoritmo que hayamos seleccionado
  cryptCreateContext(&sessionContext, CRYPT_UNUSED, ALGORITMO);
    
  //Seleccion del modo de operacion
  cryptSetAttribute(sessionContext, CRYPT_CTXINFO_MODE, MODO);

  //Generacion de clave de sesion de forma aleatoria
  cryptGenerateKey(sessionContext);
  
  //vector de inicializacion si es necesario (en todos los metodos menos ECB)
  cryptSetAttributeString(sessionContext, CRYPT_CTXINFO_IV, IV, 8);

  
  /** Exportacion de clave del contexto master al contexto de sesion **/

  //Calculo de la memoria necesaria para la exportacion
  cryptExportKey(NULL, &encryptedKeyLength, masterContext, sessionContext);

  //Exportacion
  cryptExportKey(mensaje->clave_sesion, &encryptedKeyLength, masterContext, sessionContext);

  
  /************* Cifrado de los datos ****************/
  
  //Cifra el contenido de msj con la clave y metodos asociados al contexto de sesion
  //y lo vuelve a guardar en msj
  n=strlen(msj);

  cryptEncrypt(sessionContext, msj, n);
    
  
  /************ Destruccion de contextos *****************/

  cryptDestroyContext(sessionContext);

  cryptDestroyContext(masterContext);

  /************* Fin del cifrado ***************/

  cryptEnd();

  
  /****************************************************************
   ****** Asignacion de parametros a la estructura de datos *******
   ****************************************************************/

  //mensaje
  strcpy(mensaje->msj_cifrado, msj);
  n=strlen(msj);
  mensaje->msj_cifrado[n]='\0';

  //presentacion de datos por pantalla
  printf("\n\nMensaje cifrado: %s\n", msj);

  return(0);

}



/************************ Funcion de descifrado *************************/
int descifra(datos mensaje){

  int n, status;
  char password[MAXPSWD];
  char msj[MAXMSJ];
  //Estructuras para trabajo con cryptlib
  CRYPT_CONTEXT masterContext;
  CRYPT_CONTEXT sessionContext;


  /************* pide los valores de entrada al usuario **************/
  printf("\n\n\n-------------------------------------------------");
  printf("\n\nFUNCION DE DESCIFRADO\n");

  //Contrasea
  printf("\nIntroduzca su contrasea secreta:\t");
  scanf("%s", password);
  n=strlen(password);
  password[n]=0;

  
  /******* Asignacion de parametros de la estructura de datos *********/
  strcpy(msj, mensaje.msj_cifrado);


  /********************************************************************
   ********************** Descifrado de los datos *********************
   ********************************************************************/

  /************** Inicio del descifrado ************/
  cryptInit();


  /************** Contexto Master ***************/

  //Creacion del contexto: depende del algoritmo de cifrado que queramos utilizar,
  //en este caso DES para cifrar la clave de sesion
  cryptCreateContext(&masterContext, CRYPT_UNUSED, CRYPT_ALGO_DES);

  //Derivacion de claves: obtener clave maestra a partir de la password del usuario
  //se introduce la semilla
  cryptSetAttributeString(masterContext, CRYPT_CTXINFO_KEYING_SALT, SEMILLA, 8);

  //se introduce la contrasea
  cryptSetAttributeString(masterContext, CRYPT_CTXINFO_KEYING_VALUE, password, strlen(password));

  //se introduce el vector de inicializacion si es necesario. En este caso lo es porque usamos
  //el modo por defecto (CBC), que lo necesita
  cryptSetAttributeString(masterContext, CRYPT_CTXINFO_IV, IV, 8);

  
  /*************** Contexto de sesion *************/

  //Generacion del contexto, dependera del algoritmo que hayamos seleccionado
  cryptCreateContext(&sessionContext, CRYPT_UNUSED, ALGORITMO);

  //Seleccion del modo de operacion
  cryptSetAttribute(sessionContext, CRYPT_CTXINFO_MODE, MODO);
  
  //vector de inicializacion si es necesario (en todos los metodos menos ECB)
  cryptSetAttributeString(sessionContext, CRYPT_CTXINFO_IV, IV, 8);
  

  /** Importacion de clave del contexto master al contexto de sesion **/
  cryptImportKey(mensaje.clave_sesion, masterContext, sessionContext);


  /************* Descifrado de los datos ****************/

  //Descifra el contenido de msj con la clave y metodos asociados al contexto de sesion
  //y lo vuelve a guardar en msj
  n=strlen(msj);

  cryptDecrypt(sessionContext, msj, n);


  /************ Destruccion de contextos *****************/

  cryptDestroyContext(sessionContext);

  cryptDestroyContext(masterContext);


  /************* Fin del descifrado ***************/

  cryptEnd();

  //presentacion de datos por pantalla
  printf("\n\nMensaje descifrado: %s\n\n", msj);
  
  return(0);
}
