#include <stdio.h>
#include <stdlib.h>
#include <string.h>
//biblioteca para cryptlib
#include "cryptlib.h"


#define MAXNAME 20
#define MAXMSJ 2048
#define MAXPSWD 10
#define MAXSIGN 1024
#define KPRIV "secring.skr"
#define KPUB "pubring.pkr"


struct datos{
  char nombre[MAXNAME];
  char msj[MAXMSJ];
  char firma[MAXSIGN];
};
typedef struct datos datos;


int cifra(datos *mensaje);
int descifra(datos mensaje);


/******* Funcion principal *********/

int main(int argc, char *argv[]){

  datos mensaje;

  system("clear");

  printf("Aplicacion para firmar digitalmente un mensaje\n");
  printf("-----------------------------------------------------\n");

  cifra(&mensaje);

  descifra(mensaje);

  exit(0);


}


/********************** Funcion para firmado *********************/
int cifra(datos *mensaje){

  int  n, status; 
  char password[MAXPSWD];

  //Variables para trabajo con cryptlib
  CRYPT_CONTEXT shaContext, privateKeyContext;
  CRYPT_KEYSET cryptKeyset;
  int signcliLength;
  
  
  
  printf("\n\nFUNCION PARA FIRMADO\n\n");
  

  /***************************************************************************
   **************** Peticion de datos de entrada al usuario ******************
   ***************************************************************************/
  
  //mensaje
  printf("\nIntroduzca el mensaje a firmar:\n\n");
  scanf("%[^\n]", mensaje->msj);
  n=strlen(mensaje->msj);
  mensaje->msj[n]='\0';
  
  //nombre del cliente
  printf("\nIntroduzca su nombre:\t");
  getchar();
  fflush(stdin);
  scanf("%[^\n]", mensaje->nombre);
  n=strlen(mensaje->nombre);
  mensaje->nombre[n]=0;

  //contrasea
  printf("Introduzca su contrasea:\t");
  scanf("%s", password);
  fflush(stdin);
  n=strlen(password);
  password[n]=0;
  fflush(stdin);
  

  /*******************************************************************************
   **************** Firma del mensaje con la clave privada ***********************
   *******************************************************************************/
  
  /************** Inicio de uso de cryptlib ************/
  
  status=cryptInit();
  if(cryptStatusError(status))
    printf("Init: %d\n", status);
  
  
  /********** Obtencin de la clave privada del archivo de llaves **********/
  
  //Se crea un keyset: contenedor abstracto para una o ms claves
  status=cryptKeysetOpen(&cryptKeyset, CRYPT_UNUSED, CRYPT_KEYSET_FILE, KPRIV, CRYPT_KEYOPT_READONLY);
  if(cryptStatusError(status))
    printf("Creacion de keyset: %d\n", status);
  
  /*** cryptKeySet: keyset que creamos
       CRIPT_UNUSED: usuario propietario del keyset: por defecto
       CRYPT_KEYSET_FILE: la clave est en un archivo plano en el disco duro
       CRYPT_KEYOPT_READONLY: el acceso es de slo lectura ***/
  
  //Se extrae la clave del archivo
  status=cryptGetPrivateKey(cryptKeyset, &privateKeyContext, CRYPT_KEYID_NAME, mensaje->nombre, password);
  if(status==-4 || status==-22)
    printf("Nombre o contrasea para extraccion de clave privada no validos: %d\n", status);
  
  /*** privateKeyContext: contexto de cifrado que devuelve la funcin
       CRYPT_KEYID_NAME: el propietario de la clave se especifica con el nombre (namecli) ***/
  
  //Se destruye el keyset utilizado para la clave privada
  status=cryptKeysetClose(cryptKeyset);
  if(cryptStatusError(status))
    printf("Destruccion de keyset de clave privada: %d\n", status);

 
  
  /********* Firma de los datos ***************/
  
  //Se crea un contexto para utilizar hash SHA: shaContext, con algoritmo SHA (CRYPT_ALGO_SHA)
  status=cryptCreateContext(&shaContext, CRYPT_UNUSED, CRYPT_ALGO_SHA);
  if(cryptStatusError(status))
    printf("Creacion de contexto SHA: %d\n", status);
  
  
  //Se crea el resumen hash del mensaje y se guarda en el contexto creado
  status=cryptEncrypt(shaContext, mensaje->msj, strlen(mensaje->msj));
  if(cryptStatusError(status))
    printf("Resumen hash del mensaje: %d\n", status);
  
  status=cryptEncrypt(shaContext, mensaje->msj, 0);
  if(cryptStatusError(status))
    printf("Resumen hash del mensaje: %d\n", status);
  
  //Se firma el resumen hash del mensaje utilizando el algoritmo SHA en shaCOntext
  //y la clave privada en privateKeyContext.
  
  //Se guarda la firma
  status=cryptCreateSignature(mensaje->firma, &signcliLength, privateKeyContext, shaContext);
  if(cryptStatusError(status))
    printf("Firma de datos: %d\n", status);


  /************ Destruccion de contextos *****************/
  
  status=cryptDestroyContext(shaContext);
  if(cryptStatusError(status))
    printf("Destruccion contexto SHA: %d\n", status);


  /************* Fin del uso de cryptlib***************/
  
  cryptEnd();


  //presentacion de resultado por pantalla
  printf("\nmensaje firmado: %s | %s | %s\n\n", mensaje->nombre, mensaje->msj, mensaje->firma);

  return(0);

}


/********************** Funcion para comprobacion de firma *********************/
int descifra(datos mensaje){

  int status; 
  char modificar='a';
  enum autent{SI, NO} autent;

  //Variables para trabajo con cryptlib
  CRYPT_CONTEXT shaContext;
  CRYPT_KEYSET cryptKeyset;
  CRYPT_HANDLE publicKey;

  
  printf("\n\n\n-------------------------------------------------");
  printf("\n\nFUNCION PARA COMPROBACION DE FIRMA\n\n");


  /***** Opcion para modificar los datos y comprobar que funciona el programa *****/
  modificar='a';
  while(modificar!='n' && modificar!='s'){
    printf("\nComprobacion de funcionamiento: Quiere modificar los datos recibidos (s/n)?: ");
    getchar();
    fflush(stdin);
    fflush(stdin);
    scanf("%[^\n]", &modificar);
  }
  if(modificar=='s')
    mensaje.msj[0]=(char)((int)mensaje.msj[0]+1); //cambia el primer caracter por el siguiente en el alfabeto
  fflush(stdin);

  
  /********************************************************************
   ************ Obtencin de la clave pblica del emisor **************
   ********************************************************************/
  
  /************** Inicio ************/
	  
  status=cryptInit();
  if(cryptStatusError(status))
    printf("Init: %d\n", status);
  
  //Se crea un keyset: contenedor abstracto para una o ms claves
  status=cryptKeysetOpen(&cryptKeyset, CRYPT_UNUSED, CRYPT_KEYSET_FILE, KPUB, CRYPT_KEYOPT_READONLY);
  if(cryptStatusError(status))
    printf("Creacion de keyset para clave publica: %d\n", status);

  /*** cryptKeySet: keyset que creamos
       CRIPT_UNUSED: usuario propietario del keyset: por defecto
       CRYPT_KEYSET_FILE: la clave est en un archivo plano en el disco duro
       CRYPT_KEYOPT_READONLY: el acceso es de slo lectura ***/
  
  //Se extrae la clave del archivo
  status=cryptGetPublicKey(cryptKeyset, &publicKey, CRYPT_KEYID_NAME, mensaje.nombre);
  if(cryptStatusError(status))
    printf("Extraccion de clave publica: %d\n", status);
  
  /*** publicKey: contexto en el que se guarda la clave
       CRYPT_KEYID_NAME: el propietario de la clave se especifica con el nombre (namecli) ***/
  
  

  /*******************************************************************
   ************** Comprobacin de la firma del emisor ****************
   *******************************************************************/
  
  //Se crea un contexto para la firma
  status=cryptCreateContext(&shaContext, CRYPT_UNUSED, CRYPT_ALGO_SHA);
  if(cryptStatusError(status))
    printf("Creacion contexto SHA: %d\n", status);
  
  //Se crea el resumen hash del mensaje y se guarda en el contexto creado
  status=cryptEncrypt(shaContext, mensaje.msj, strlen(mensaje.msj));
  if(cryptStatusError(status))
    printf("Resumen hash del mensaje: %d\n", status);
  
  status=cryptEncrypt(shaContext, mensaje.msj, 0);
  if(cryptStatusError(status))
    printf("Resumen hash del mensaje: %d\n", status);
  
  
  //Se comprueba la firma del cliente
  status=cryptCheckSignature(mensaje.firma, publicKey, shaContext);
  if(cryptStatusOK(status)){
    autent=SI;
  }
  else
    autent=NO;
    
  
  /************ Destruccion de contextos *****************/
  
  status=cryptDestroyContext(shaContext);
  if(cryptStatusError(status))
    printf("Destruccion contexto SHA: %d\n", status);
  
  //Se destruye el keyset utilizado para la clave pblica
  status=cryptKeysetClose(cryptKeyset);
  if(cryptStatusError(status))
    printf("Destruccion de keyset de clave publica: %d\n", status);
  
  
  /********************************************************************
   ****************** Visualizacin de resultados *********************
   ********************************************************************/
  
  //visualiza el mensaje recibido
  printf("\n\nRemitente: %s\n", mensaje.nombre);
  printf("\nMensaje recibido: %s\n", mensaje.msj);
  fflush(stdout);
  
  if(autent==SI)
    printf("\nComprobacion de firma correcta\n\n\n");
  else
    printf("\nComprobacion de firma incorrecta\n\n\n");
  

  /************* Fin del uso de cryptlib***************/
  cryptEnd();

  return(0);
  
}
