#include <stdio.h>
#include <stdlib.h>
#include <string.h>
//biblioteca para cryptlib
#include "cryptlib.h"

#define MAXNAME 20
#define MAXMSJ 2048
#define MAXPSWD 10
#define MAXSIGN 1024
#define KPRIV "secring.skr"
#define KPUB "pubring.pkr"

struct datos{
  char nombre[MAXNAME];
  char msj[MAXMSJ];
  char firma[MAXSIGN];
};
typedef struct datos datos;

//definicion de funciones para firma y comprobacion
int cifra(datos *mensaje);
int descifra(datos mensaje);


/******* Funcion principal *********/
int main(int argc, char *argv[]){

  datos mensaje;

  system("clear");

  printf("Aplicacion para firmar digitalmente un mensaje\n");
  printf("-----------------------------------------------------\n");

  cifra(&mensaje);

  descifra(mensaje);

  exit(0);
}


/********************** Funcion para firmado *********************/
int cifra(datos *mensaje){

  int  n, status; 
  char password[MAXPSWD];
  //Variables para trabajo con cryptlib
  CRYPT_CONTEXT shaContext, privateKeyContext;
  CRYPT_KEYSET cryptKeyset;
  int signcliLength;
  
  printf("\n\nFUNCION PARA FIRMADO\n\n");
  
  /*********** datos de entrada *************/
  
  //mensaje
  printf("\nIntroduzca el mensaje a firmar:\n\n");
  scanf("%[^\n]", mensaje->msj);
  n=strlen(mensaje->msj);
  mensaje->msj[n]='\0';
  
  //nombre del cliente
  printf("\nIntroduzca su nombre:\t");
  getchar();
  fflush(stdin);
  scanf("%[^\n]", mensaje->nombre);
  n=strlen(mensaje->nombre);
  mensaje->nombre[n]=0;

  //contrasea
  printf("Introduzca su contrasea:\t");
  scanf("%s", password);
  n=strlen(password);
  password[n]=0;


  /*******************************************************************************
   **************** Firma del mensaje con la clave privada ***********************
   *******************************************************************************/
  
  /************** Inicio de uso de cryptlib ************/
  cryptInit();

  
  /********** Obtencin de la clave privada del archivo de llaves **********/
  
  //Se crea un keyset: contenedor abstracto para una o ms claves
  cryptKeysetOpen(&cryptKeyset, CRYPT_UNUSED, CRYPT_KEYSET_FILE, KPRIV, CRYPT_KEYOPT_READONLY);
  
  /*** cryptKeySet: keyset que creamos
       CRIPT_UNUSED: usuario propietario del keyset: por defecto
       CRYPT_KEYSET_FILE: la clave est en un archivo plano en el disco duro
       CRYPT_KEYOPT_READONLY: el acceso es de slo lectura ***/
  
  //Se extrae la clave del archivo
  cryptGetPrivateKey(cryptKeyset, &privateKeyContext, CRYPT_KEYID_NAME, mensaje->nombre, password);
  
  /*** privateKeyContext: contexto de cifrado que devuelve la funcin
       CRYPT_KEYID_NAME: el propietario de la clave se especifica con el nombre (mensaje->nombre) ***/
  

  /********* Firma de los datos ***************/
  
  //Se crea un contexto para utilizar hash SHA: shaContext, con algoritmo SHA (CRYPT_ALGO_SHA)
  cryptCreateContext(&shaContext, CRYPT_UNUSED, CRYPT_ALGO_SHA);
  
  //Se crea el resumen hash del mensaje y se guarda en el contexto creado
  cryptEncrypt(shaContext, mensaje->msj, strlen(mensaje->msj));
  
  cryptEncrypt(shaContext, mensaje->msj, 0);
  
  //Se firma el resumen hash del mensaje utilizando el algoritmo SHA en shaCOntext
  //y la clave privada en privateKeyContext.
  cryptCreateSignature(mensaje->firma, &signcliLength, privateKeyContext, shaContext);


  /************ Destruccion de contextos *****************/
  cryptDestroyContext(shaContext);
    
  //Se destruye el keyset utilizado para la clave privada
  cryptKeysetClose(cryptKeyset);


  /************* Fin del uso de cryptlib***************/
  cryptEnd();


  //presentacion de resultado por pantalla
  printf("\nmensaje firmado: %s | %s | %s\n\n", mensaje->nombre, mensaje->msj, mensaje->firma);

  return(0);
}


/********************** Funcion para comprobacion de firma *********************/
int descifra(datos mensaje){

  int status; 
  char modificar='a';
  enum autent{SI, NO} autent;
  //Variables para trabajo con cryptlib
  CRYPT_CONTEXT shaContext;
  CRYPT_KEYSET cryptKeyset;
  CRYPT_HANDLE publicKey;

  printf("\n\n\n-------------------------------------------------");
  printf("\n\nFUNCION PARA COMPROBACION DE FIRMA\n\n");


  /***** Opcion para modificar los datos y comprobar que funciona el programa *****/
  modificar='a';
  while(modificar!='n' && modificar!='s'){
    printf("\nComprobacion de funcionamiento: Quiere modificar los datos recibidos (s/n)?: ");
    getchar();
    fflush(stdin);
    scanf("%[^\n]", &modificar);
  }
  if(modificar=='s')
    mensaje.msj[0]=(char)((int)mensaje.msj[0]+1); //cambia el primer caracter por el siguiente en el alfabeto

  
  /********************************************************************
   ************ Comprobacion de la firma del emisor **** **************
   ********************************************************************/
  
  /************** Inicio ************/
  cryptInit();
  
  
  /************ Obtencion de la clave publica del emisor ************/
  
  //Se crea un keyset: contenedor abstracto para una o ms claves
  cryptKeysetOpen(&cryptKeyset, CRYPT_UNUSED, CRYPT_KEYSET_FILE, KPUB, CRYPT_KEYOPT_READONLY);

  /*** cryptKeySet: keyset que creamos
       CRIPT_UNUSED: usuario propietario del keyset: por defecto
       CRYPT_KEYSET_FILE: la clave est en un archivo plano en el disco duro
       CRYPT_KEYOPT_READONLY: el acceso es de slo lectura ***/
  
  //Se extrae la clave del archivo
  cryptGetPublicKey(cryptKeyset, &publicKey, CRYPT_KEYID_NAME, mensaje.nombre);
  
  /*** publicKey: contexto en el que se guarda la clave
       CRYPT_KEYID_NAME: el propietario de la clave se especifica con el nombre (mensaje.nombre) ***/
  
  
  /************* Comprobacin de la firma del emisor ****************/
  
  //Se crea un contexto para la firma
  cryptCreateContext(&shaContext, CRYPT_UNUSED, CRYPT_ALGO_SHA);
  
  //Se crea el resumen hash del mensaje y se guarda en el contexto creado
  cryptEncrypt(shaContext, mensaje.msj, strlen(mensaje.msj));
  
  cryptEncrypt(shaContext, mensaje.msj, 0);
  
  //Se comprueba la firma del emisor
  status=cryptCheckSignature(mensaje.firma, publicKey, shaContext);
  if(cryptStatusOK(status)){
    autent=SI;
  }
  else
    autent=NO;
    
  
  /************ Destruccion de contextos *****************/
  cryptDestroyContext(shaContext);
  
  //Se destruye el keyset utilizado para la clave pblica
  cryptKeysetClose(cryptKeyset);

  /************* Fin del uso de cryptlib***************/
  cryptEnd();
  
  
  /************* Visualizacin de resultados *************/
  
  //visualiza el mensaje recibido
  printf("\n\nRemitente: %s\n", mensaje.nombre);
  printf("\nMensaje recibido: %s\n", mensaje.msj);
  
  if(autent==SI)
    printf("\nComprobacion de firma correcta\n\n\n");
  else
    printf("\nComprobacion de firma incorrecta\n\n\n");
  
  return(0);
}
